from pyicubsim import iCubRightArm, iCubLeftArm, iCubTorso, iCubHead, iCubBall, iCubLeftEye
import numpy as np
import cv2 as cv
import time
import os
import signal

# exit on ctrl-c
def signal_handler(signal, frame):
    os._exit(0)

signal.signal(signal.SIGINT, signal_handler)

#initialize
rightArm = iCubRightArm()
leftArm = iCubLeftArm()
head = iCubHead()
torso = iCubTorso()

def pick_up():
    torso.set(joint0=-1, joint1=0.6, joint2=23.20)
    rightArm.set(joint0=-52.5, joint1=0, joint2=5)
    leftArm.set(joint0=-52.5, joint1=0, joint2=5)
    time.sleep(1.2)
    torso.set((0,0,0))

def set_standard_positions():
    standard_arm_position = (0, 80, 0, 50, 0, 0, 0, 59, 20, 20, 20, 10, 10, 10, 10, 10)
    rightArm.set(standard_arm_position)
    leftArm.set(standard_arm_position)
    head.set((-37.80, 0, -8.800, 0, 0, 0))
    torso.set(joint0=-1, joint1=0.6, joint2=23.20)
    time.sleep(1.5)

ball = iCubBall()
ball.set((-300.0, 0.0, -46.0245))

set_standard_positions()

def blue_filter(bgr):
    # converting from BGR to HSV color space
    hsv = cv.cvtColor(bgr, cv.COLOR_BGR2HSV)
    # Hue range for blue derived from its BGR
    blue = np.uint8([[[80, 0, 0]]])
    hsvBlue = cv.cvtColor(blue,cv.COLOR_BGR2HSV)
    lowerLimit = np.array([hsvBlue[0][0][0]-10,100,25],np.uint8)
    upperLimit = np.array([hsvBlue[0][0][0]+10,255,255],np.uint8)
    # filtering in HSV
    mask = cv.inRange(hsv, lowerLimit, upperLimit)
    # redraw the detected ball
    indices = np.where(mask > 0)
    bgr[indices[0],indices[1],:] = [255,255,0]
    # if pixel number of ball is too small, it is perhaps not the ball at all
    if np.shape(indices)[1] < 500:
        # ball not found
        return None
    else:
        # return average of x and y coordinates of blue pixels
        return (int(np.average(indices[1])),int(np.average(indices[0])))
        
def speak(text):
    print(text)
    filename = str(time.time())+'.txt'
    file = open(filename,'wb') 
    file.write(bytes(text,'utf-16')) 
    file.close() 
    os.environ["ESPEAK_DATA_PATH"] = "." 
    os.system('espeak -v EN -b4 -f '+filename)
    os.remove(filename)

leftEye = iCubLeftEye()
while True:
    frame = leftEye.grab()
    
    pos = blue_filter(frame)
    if pos is not None:
        print(pos)
        cv.circle(frame,pos,4,(0,0,255),cv.FILLED)
        if np.linalg.norm(np.array(pos)-np.array((148,110))) < 3.0:
            pick_up()
            speak("the ball")
            set_standard_positions()
                
    cv.imshow("leftEye",frame)
    if cv.waitKey(1) == 27:
        break

cv.destroyAllWindows()
