from pyicubsim import iCubRightArm, iCubLeftArm, iCubTorso, iCubHead, iCubBall, iCubLeftEye, iCubEmotion
import numpy as np
import cv2 as cv
from agentspace import space, Agent
import time
import os
import signal

# exit on ctrl-c
def signal_handler(signal, frame):
    os._exit(0)

signal.signal(signal.SIGINT, signal_handler)

#initialize
rightArm = iCubRightArm()
leftArm = iCubLeftArm()
head = iCubHead()
torso = iCubTorso()
emotion = iCubEmotion()

ball = iCubBall()
ball.set((-300.0, 0.0, -46.0245))

class CameraAgent(Agent):

    def init(self):
        leftEye = iCubLeftEye()
        while True:
            frame = leftEye.grab()
            space["bgr"] = frame
            cv.imshow("leftEye",frame)
            cv.waitKey(1)
            
    def senseSelectAct(self):
        pass

class BallDetectorAgent(Agent):

    def blue_filter(self, bgr):
        # converting from BGR to HSV color space
        hsv = cv.cvtColor(bgr, cv.COLOR_BGR2HSV)
        # Hue range for blue derived from its BGR
        blue = np.uint8([[[80, 0, 0]]])
        hsvBlue = cv.cvtColor(blue,cv.COLOR_BGR2HSV)
        lowerLimit = np.array([hsvBlue[0][0][0]-10,100,25],np.uint8)
        upperLimit = np.array([hsvBlue[0][0][0]+10,255,255],np.uint8)
        # filtering in HSV
        mask = cv.inRange(hsv, lowerLimit, upperLimit)
        # redraw the detected ball
        indices = np.where(mask > 0)
        bgr[indices[0],indices[1],:] = [255,255,0]
        # if pixel number of ball is too small, it is perhaps not the ball at all
        if np.shape(indices)[1] < 500:
            # ball not found
            return None
        else:
            # return average of x and y coordinates of blue pixels
            return (int(np.average(indices[1])),int(np.average(indices[0])))
            
    def init(self):
        space.attach_trigger("bgr",self)
        
    def senseSelectAct(self):
        frame = space["bgr"]
        if frame is None:
            return
        pos = self.blue_filter(frame)
        if pos is not None:
            #print(pos)
            cv.circle(frame,pos,4,(0,0,255),cv.FILLED)
            space(validity=0.1)["pos"] = pos
        cv.imshow("SeenByTheleftEye",frame)
        cv.waitKey(1)

class ControlAgent(Agent):

    def speak(self,text):
        print(text)
        filename = str(time.time())+'.txt'
        file = open(filename,'wb') 
        file.write(bytes(text,'utf-16')) 
        file.close() 
        os.environ["ESPEAK_DATA_PATH"] = "." 
        os.system('espeak -v EN -b4 -f '+filename)
        os.remove(filename)

    def pick_up(self):
        torso.set(joint0=-1, joint1=0.6, joint2=23.20)
        rightArm.set(joint0=-52.5, joint1=0, joint2=5)
        leftArm.set(joint0=-52.5, joint1=0, joint2=5)
        time.sleep(1.2)
        torso.set((0,0,0))

    def set_standard_positions(self):
        standard_arm_position = (0, 80, 0, 50, 0, 0, 0, 59, 20, 20, 20, 10, 10, 10, 10, 10)
        rightArm.set(standard_arm_position)
        leftArm.set(standard_arm_position)
        head.set((-37.80, 0, -8.800, 0, 0, 0))
        torso.set(joint0=-1, joint1=0.6, joint2=23.20)
        time.sleep(1.5)  

    def init(self):
        self.set_standard_positions()
        space.attach_trigger("pos",self)

    def senseSelectAct(self):
        pos = space["pos"]
        if pos is None:
            return
        if np.linalg.norm(np.array(pos)-np.array((148,110))) < 3.0:
            self.pick_up()
            h = head.get()[0]
            head.set(joint0=h+45)
            time.sleep(0.8)
            space["speaking"] = True
            self.speak("the ball")
            space["speaking"] = False
            self.set_standard_positions()

class LipsAgent(Agent):

    def init(self):
        self.active = False
        emotion.set('hap')
        self.attach_timer(0.15)
        
    def senseSelectAct(self):
        speaking = space(default=False)["speaking"]
        if speaking:
            if self.active:
                emotion.set('hap')
                self.active = False
            else: 
                emotion.set('cun')
                self.active = True
        else:
            if self.active:
                emotion.set('hap')
                self.active = False
        
CameraAgent()
BallDetectorAgent()
ControlAgent()
LipsAgent()
# use W to zoom onto the robot